// -*- C++ -*-
//
// $Id: SL_Disabled_Receiver_exec.cpp 92187 2010-10-08 10:06:17Z johnnyw $

// Test for statelistener: NOT_ENABLED

#include "SL_Disabled_Receiver_exec.h"
#include "ace/Log_Msg.h"
#include "tao/ORB_Core.h"
#include "ace/OS_NS_time.h"
#include "ace/Reactor.h"
#include "dds4ccm/impl/Utils.h"
#include "dds4ccm/impl/TimeUtilities.h"

#define SAMPLES_EXPECTED 3

namespace CIAO_SL_Disabled_Receiver_Impl
{
  //============================================================
  // read_action_Generator
  //============================================================
  read_action_Generator::read_action_Generator (Receiver_exec_i &callback)
    : pulse_callback_ (callback)
  {
  }

  read_action_Generator::~read_action_Generator ()
  {
  }

  int
  read_action_Generator::handle_timeout (const ACE_Time_Value &, const void *)
  {
    try
      {
        this->pulse_callback_.read_all();
      }
    catch (...)
      {
        // @todo
      }
    return 0;
  }

  //============================================================
  // StateListener_exec_i
  //============================================================
  StateListener_exec_i::StateListener_exec_i (Atomic_Long & no_operation)
    : no_operation_ (no_operation)
  {
  }

  StateListener_exec_i::~StateListener_exec_i (void)
  {
  }

  // Operations from ::CCM_DDS::StateListener
  void
  StateListener_exec_i::on_creation (const ::TestTopic & /*datum*/,
                                     const ::CCM_DDS::ReadInfo & /*info*/)
  {
    ++this->no_operation_;
  }

  void
  StateListener_exec_i::on_one_update (const ::TestTopic & /*datum*/,
                                       const ::CCM_DDS::ReadInfo & /*info*/)
  {
    ++this->no_operation_;
  }

  void
  StateListener_exec_i::on_many_updates (const ::TestTopicSeq & /*data*/,
                                         const ::CCM_DDS::ReadInfoSeq & /*infos*/)
  {
    ++this->no_operation_;
  }

  void
  StateListener_exec_i::on_deletion (const ::TestTopic & /*datum*/,
                                     const ::CCM_DDS::ReadInfo & /*info*/)
  {
    ++this->no_operation_;
  }

  //============================================================
  // Receiver_exec_i
  //============================================================
  Receiver_exec_i::Receiver_exec_i (void)
  : rate_ (10),
    no_operation_ (0),
    updater_data_ (0)
  {
    this->ticker_ = new read_action_Generator (*this);
  }

  Receiver_exec_i::~Receiver_exec_i (void)
  {
  }

  // Supported operations and attributes.
  void
  Receiver_exec_i::read_all (void)
  {
    ::SL_Disabled::SL_DisabledConnector::Reader_var reader =
      this->context_->get_connection_info_out_data ();

    if (::CORBA::is_nil (reader.in ()))
      {
        return;
      }
    try
      {
        TestTopicSeq TestTopic_infos;
        ::CCM_DDS::ReadInfoSeq readinfoseq;
        reader->read_all(TestTopic_infos, readinfoseq);
        for (CORBA::ULong i = 0; i < readinfoseq.length(); ++i)
          {
            ACE_Time_Value tv;
            tv <<= readinfoseq[i].source_timestamp;
            ACE_DEBUG ((LM_DEBUG, ACE_TEXT ("READ_ALL ReadInfo ")
                                  ACE_TEXT ("-> UTC date =%#T\n"),
                                  &tv));
          }
        for (CORBA::ULong i = 0; i < TestTopic_infos.length(); ++i)
          {
            ACE_DEBUG ((LM_DEBUG, ACE_TEXT ("READ_ALL keyed test info : ")
                       ACE_TEXT ("Number <%d> : received TestTopic_info for <%C> at %u\n"),
                        i,
                        TestTopic_infos[i].key.in (),
                        TestTopic_infos[i].x));
          }
        this->updater_data_ += readinfoseq.length ();
      }
    catch (const CCM_DDS::InternalError& )
      {
        ACE_DEBUG ((LM_DEBUG, ACE_TEXT ("internal error or no data\n")));
      }
  }

  // Port operations.
  ::CCM_DDS::CCM_PortStatusListener_ptr
  Receiver_exec_i::get_info_out_status (void)
  {
    ACE_DEBUG ((LM_DEBUG, ACE_TEXT ("new PortStatuslistener\n")));
    return ::CCM_DDS::CCM_PortStatusListener::_nil ();
  }

  ::SL_Disabled::SL_DisabledConnector::CCM_StateListener_ptr
  Receiver_exec_i::get_info_out_data_listener (void)
  {
     return new StateListener_exec_i(this->no_operation_);
  }

  // Operations from Components::SessionComponent.
  void
  Receiver_exec_i::set_session_context (
    ::Components::SessionContext_ptr ctx)
  {
    this->context_ =
      ::SL_Disabled::CCM_Receiver_Context::_narrow (ctx);
    if ( ::CORBA::is_nil (this->context_.in ()))
      {
        throw ::CORBA::INTERNAL ();
      }
  }

  void
  Receiver_exec_i::configuration_complete (void)
  {
  }

  void
  Receiver_exec_i::ccm_activate (void)
  {
    ::CCM_DDS::StateListenerControl_var lc =
    this->context_->get_connection_info_out_data_control ();

    if (::CORBA::is_nil (lc.in ()))
      {
         ACE_ERROR ((LM_INFO, ACE_TEXT ("Error:  Listener control receptacle is null!\n")));
        throw ::CORBA::INTERNAL ();
      }

    lc->mode (::CCM_DDS::NOT_ENABLED);

    // Calculate the interval time
    long const usec = 1000000 / this->rate_;
    if (this->context_->get_CCM_object()->_get_orb ()->orb_core ()->reactor ()->schedule_timer (
                                          this->ticker_,
                                          0,
                                          ACE_Time_Value(3, usec),
                                          ACE_Time_Value(3, usec)) == -1)
      {
        ACE_ERROR ((LM_ERROR, "Unable to schedule Timer\n"));
      }
  }

  void
  Receiver_exec_i::ccm_passivate (void)
  {
    this->context_->get_CCM_object()->_get_orb ()->orb_core ()->reactor ()->cancel_timer (this->ticker_);
    delete this->ticker_;
  }

  void
  Receiver_exec_i::ccm_remove (void)
  {
    if (this->no_operation_.value () > 0)
      {

         ACE_ERROR ((LM_ERROR,
                     ACE_TEXT ("ERROR: Received an unexpected ")
                     ACE_TEXT (" operation. StateListener doesn't work in Receiver\n")
                    ));
      }

    else
      {
        ACE_DEBUG ((LM_DEBUG,
                    ACE_TEXT ("OK : Haven't received an  unexpected ")
                    ACE_TEXT (" oparation from StateListener in Receiver\n")
                   ));
      }
    if (this->updater_data_.value () != SAMPLES_EXPECTED)
      {

         ACE_ERROR ((LM_ERROR,
                     ACE_TEXT ("ERROR: Unexpected number of samples received: ")
                     ACE_TEXT ("expected <%d> - received <%d>\n"),
                     SAMPLES_EXPECTED, this->updater_data_.value ()
                    ));
      }

    else
      {
        ACE_DEBUG ((LM_DEBUG,
                    ACE_TEXT ("OK : Received the expected number of samples: ")
                    ACE_TEXT ("expected <%d> - received <%d>\n"),
                    SAMPLES_EXPECTED, this->updater_data_.value ()
                    ));
      }
  }

  extern "C" RECEIVER_EXEC_Export ::Components::EnterpriseComponent_ptr
  create_SL_Disabled_Receiver_Impl (void)
  {
    ::Components::EnterpriseComponent_ptr retval =
      ::Components::EnterpriseComponent::_nil ();

    ACE_NEW_NORETURN (
      retval,
      Receiver_exec_i);

    return retval;
  }
}
